@@ -12,8 +12,8 @@
 #include "appletquickitem.h"
 #include "config-plasma.h"
 #include "configview.h"
+#include "dialogbackground_p.h"
 #include "dialogshadows_p.h"
-#include "private/dialogbackground_p.h"
 #include "sharedqmlengine.h"
 
 #include <QLayout>
