@@ -106,12 +106,6 @@ void PlasmoidItem::init()
         bool activate = !(isExpanded() && isActivationTogglesExpanded());
 
         setExpanded(activate);
-
-        // Focus any container, such as panels
-        if (Plasma::Containment *y = qobject_cast<Plasma::Containment *>(applet)) {
-            y->setStatus(Plasma::Types::AcceptingInputStatus);
-            return;
-        }
     });
 
     connect(applet, &Plasma::Applet::destroyedChanged, this, &PlasmoidItem::destroyedChanged);
